@@ -14,6 +14,7 @@ from PyQt4 import QtCore
 
 import constants
 from util import qtpie
+from util import qwrappers
 from util import qui_utils
 from util import misc as misc_utils
 
@@ -46,8 +47,7 @@ class LedWrapper(object):
 			self._ledHandler = None
 
 
-
-class Dialcentral(object):
+class Dialcentral(qwrappers.ApplicationWrapper):
 
 	_DATA_PATHS = [
 		os.path.join(os.path.dirname(__file__), "../share"),
@@ -55,45 +55,13 @@ class Dialcentral(object):
 	]
 
 	def __init__(self, app):
-		self._app = app
-		self._recent = []
-		self._hiddenCategories = set()
-		self._hiddenUnits = {}
-		self._clipboard = QtGui.QApplication.clipboard()
 		self._dataPath = None
+		self._aboutDialog = None
 		self._ledHandler = LedWrapper()
 		self.notifyOnMissed = False
 		self.notifyOnVoicemail = False
 		self.notifyOnSms = False
 
-		self._mainWindow = None
-
-		self._fullscreenAction = QtGui.QAction(None)
-		self._fullscreenAction.setText("Fullscreen")
-		self._fullscreenAction.setCheckable(True)
-		self._fullscreenAction.setShortcut(QtGui.QKeySequence("CTRL+Enter"))
-		self._fullscreenAction.toggled.connect(self._on_toggle_fullscreen)
-
-		self._orientationAction = QtGui.QAction(None)
-		self._orientationAction.setText("Orientation")
-		self._orientationAction.setCheckable(True)
-		self._orientationAction.setShortcut(QtGui.QKeySequence("CTRL+o"))
-		self._orientationAction.toggled.connect(self._on_toggle_orientation)
-
-		self._logAction = QtGui.QAction(None)
-		self._logAction.setText("Log")
-		self._logAction.setShortcut(QtGui.QKeySequence("CTRL+l"))
-		self._logAction.triggered.connect(self._on_log)
-
-		self._quitAction = QtGui.QAction(None)
-		self._quitAction.setText("Quit")
-		self._quitAction.setShortcut(QtGui.QKeySequence("CTRL+q"))
-		self._quitAction.triggered.connect(self._on_quit)
-
-		self._app.lastWindowClosed.connect(self._on_app_quit)
-		self._mainWindow = MainWindow(None, self)
-		self._mainWindow.window.destroyed.connect(self._on_child_close)
-
 		try:
 			import alarm_handler
 			if alarm_handler.AlarmHandler is not alarm_handler._NoneAlarmHandler:
@@ -108,14 +76,7 @@ class Dialcentral(object):
 		if self._alarmHandler is None:
 			_moduleLogger.info("No notification support")
 
-		self.load_settings()
-
-		self._mainWindow.show()
-		self._idleDelay = QtCore.QTimer()
-		self._idleDelay.setSingleShot(True)
-		self._idleDelay.setInterval(0)
-		self._idleDelay.timeout.connect(lambda: self._mainWindow.start())
-		self._idleDelay.start()
+		qwrappers.ApplicationWrapper.__init__(self, app, constants)
 
 	def load_settings(self):
 		try:
@@ -232,26 +193,15 @@ class Dialcentral(object):
 		else:
 			return None
 
+	def _close_windows(self):
+		qwrappers.ApplicationWrapper._close_windows(self)
+		if self._aboutDialog  is not None:
+			self._aboutDialog.close()
+
 	@property
 	def fsContactsPath(self):
 		return os.path.join(constants._data_path_, "contacts")
 
-	@property
-	def fullscreenAction(self):
-		return self._fullscreenAction
-
-	@property
-	def orientationAction(self):
-		return self._orientationAction
-
-	@property
-	def logAction(self):
-		return self._logAction
-
-	@property
-	def quitAction(self):
-		return self._quitAction
-
 	@property
 	def alarmHandler(self):
 		return self._alarmHandler
@@ -260,62 +210,18 @@ class Dialcentral(object):
 	def ledHandler(self):
 		return self._ledHandler
 
-	def _walk_children(self):
-		if self._mainWindow is not None:
-			return (self._mainWindow, )
-		else:
-			return ()
-
-	def _close_windows(self):
-		if self._mainWindow is not None:
-			self.save_settings()
-			self._mainWindow.window.destroyed.disconnect(self._on_child_close)
-			self._mainWindow.close()
-			self._mainWindow = None
+	def _new_main_window(self):
+		return MainWindow(None, self)
 
 	@QtCore.pyqtSlot()
 	@QtCore.pyqtSlot(bool)
 	@misc_utils.log_exception(_moduleLogger)
-	def _on_app_quit(self, checked = False):
-		if self._mainWindow is not None:
-			self.save_settings()
-			self._mainWindow.destroy()
-
-	@QtCore.pyqtSlot(QtCore.QObject)
-	@misc_utils.log_exception(_moduleLogger)
-	def _on_child_close(self, obj = None):
-		if self._mainWindow is not None:
-			self.save_settings()
-			self._mainWindow = None
-
-	@QtCore.pyqtSlot()
-	@QtCore.pyqtSlot(bool)
-	@misc_utils.log_exception(_moduleLogger)
-	def _on_toggle_fullscreen(self, checked = False):
-		for window in self._walk_children():
-			window.set_fullscreen(checked)
-
-	@QtCore.pyqtSlot()
-	@QtCore.pyqtSlot(bool)
-	@misc_utils.log_exception(_moduleLogger)
-	def _on_toggle_orientation(self, checked = False):
-		for window in self._walk_children():
-			window.set_orientation(checked)
-
-	@QtCore.pyqtSlot()
-	@QtCore.pyqtSlot(bool)
-	@misc_utils.log_exception(_moduleLogger)
-	def _on_log(self, checked = False):
-		with open(constants._user_logpath_, "r") as f:
-			logLines = f.xreadlines()
-			log = "".join(logLines)
-			self._clipboard.setText(log)
-
-	@QtCore.pyqtSlot()
-	@QtCore.pyqtSlot(bool)
-	@misc_utils.log_exception(_moduleLogger)
-	def _on_quit(self, checked = False):
-		self._close_windows()
+	def _on_about(self, checked = True):
+		with qui_utils.notify_error(self._errorLog):
+			if self._aboutDialog is None:
+				import dialogs
+				self._aboutDialog = dialogs.AboutDialog(self)
+			response = self._aboutDialog.run(self._mainWindow.window)
 
 
 class DelayedWidget(object):
@@ -388,7 +294,7 @@ def _tab_factory(tab, app, session, errorLog):
 	return gv_views.__dict__[tab](app, session, errorLog)
 
 
-class MainWindow(object):
+class MainWindow(qwrappers.WindowWrapper):
 
 	KEYPAD_TAB = 0
 	RECENT_TAB = 1
@@ -430,10 +336,10 @@ class MainWindow(object):
 	assert len(_TAB_SETTINGS_NAMES) == MAX_TABS
 
 	def __init__(self, parent, app):
-		self._app = app
-
-		self._errorLog = qui_utils.QErrorLog()
-		self._errorDisplay = qui_utils.ErrorDisplay(self._errorLog)
+		qwrappers.WindowWrapper.__init__(self, parent, app)
+		self._window.setWindowTitle("%s" % constants.__pretty_app_name__)
+		self._freezer = qwrappers.AutoFreezeWindowFeature(self._app, self._window)
+		self._errorLog = self._app.errorLog
 
 		self._session = session.Session(self._errorLog, constants._data_path_)
 		self._session.error.connect(self._on_session_error)
@@ -447,7 +353,6 @@ class MainWindow(object):
 		self._credentialsDialog = None
 		self._smsEntryDialog = None
 		self._accountDialog = None
-		self._aboutDialog = None
 
 		self._tabsContents = [
 			DelayedWidget(self._app, self._TAB_SETTINGS_NAMES[i])
@@ -483,21 +388,8 @@ class MainWindow(object):
 		self._tabWidget.currentChanged.connect(self._on_tab_changed)
 		self._tabWidget.setContentsMargins(0, 0, 0, 0)
 
-		self._layout = QtGui.QVBoxLayout()
-		self._layout.setContentsMargins(0, 0, 0, 0)
-		self._layout.addWidget(self._errorDisplay.toplevel)
 		self._layout.addWidget(self._tabWidget)
 
-		centralWidget = QtGui.QWidget()
-		centralWidget.setLayout(self._layout)
-		centralWidget.setContentsMargins(0, 0, 0, 0)
-
-		self._window = QtGui.QMainWindow(parent)
-		self._window.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
-		qui_utils.set_stackable(self._window, True)
-		self._window.setWindowTitle("%s" % constants.__pretty_app_name__)
-		self._window.setCentralWidget(centralWidget)
-
 		self._loginTabAction = QtGui.QAction(None)
 		self._loginTabAction.setText("Login")
 		self._loginTabAction.triggered.connect(self._on_login_requested)
@@ -515,54 +407,19 @@ class MainWindow(object):
 		self._refreshTabAction.setShortcut(QtGui.QKeySequence("CTRL+r"))
 		self._refreshTabAction.triggered.connect(self._on_refresh)
 
-		self._aboutAction = QtGui.QAction(None)
-		self._aboutAction.setText("About")
-		self._aboutAction.triggered.connect(self._on_about)
+		fileMenu = self._window.menuBar().addMenu("&File")
+		fileMenu.addAction(self._loginTabAction)
+		fileMenu.addAction(self._refreshTabAction)
 
-		self._closeWindowAction = QtGui.QAction(None)
-		self._closeWindowAction.setText("Close")
-		self._closeWindowAction.setShortcut(QtGui.QKeySequence("CTRL+w"))
-		self._closeWindowAction.triggered.connect(self._on_close_window)
-
-		if constants.IS_MAEMO:
-			fileMenu = self._window.menuBar().addMenu("&File")
-			fileMenu.addAction(self._loginTabAction)
-			fileMenu.addAction(self._refreshTabAction)
-
-			toolsMenu = self._window.menuBar().addMenu("&Tools")
-			toolsMenu.addAction(self._accountTabAction)
-			toolsMenu.addAction(self._importTabAction)
-			toolsMenu.addAction(self._aboutAction)
-
-			self._window.addAction(self._closeWindowAction)
-			self._window.addAction(self._app.quitAction)
-			self._window.addAction(self._app.fullscreenAction)
-		else:
-			fileMenu = self._window.menuBar().addMenu("&File")
-			fileMenu.addAction(self._loginTabAction)
-			fileMenu.addAction(self._refreshTabAction)
-			fileMenu.addAction(self._closeWindowAction)
-			fileMenu.addAction(self._app.quitAction)
-
-			viewMenu = self._window.menuBar().addMenu("&View")
-			viewMenu.addAction(self._app.fullscreenAction)
-
-			toolsMenu = self._window.menuBar().addMenu("&Tools")
-			toolsMenu.addAction(self._accountTabAction)
-			toolsMenu.addAction(self._importTabAction)
-			toolsMenu.addAction(self._aboutAction)
-
-		self._window.addAction(self._app.orientationAction)
-		self._window.addAction(self._app.logAction)
+		toolsMenu = self._window.menuBar().addMenu("&Tools")
+		toolsMenu.addAction(self._accountTabAction)
+		toolsMenu.addAction(self._importTabAction)
+		toolsMenu.addAction(self._app.aboutAction)
 
 		self._initialize_tab(self._tabWidget.currentIndex())
 		self.set_fullscreen(self._app.fullscreenAction.isChecked())
 		self.set_orientation(self._app.orientationAction.isChecked())
 
-	@property
-	def window(self):
-		return self._window
-
 	def set_default_credentials(self, username, password):
 		self._defaultCredentials = username, password
 
@@ -573,6 +430,7 @@ class MainWindow(object):
 		return ()
 
 	def start(self):
+		qwrappers.WindowWrapper.start(self)
 		assert self._session.state == self._session.LOGGEDOUT_STATE, "Initialization messed up"
 		if self._defaultCredentials != ("", ""):
 			username, password = self._defaultCredentials[0], self._defaultCredentials[1]
@@ -582,20 +440,17 @@ class MainWindow(object):
 			self._prompt_for_login()
 
 	def close(self):
-		for child in self.walk_children():
-			child.window.destroyed.disconnect(self._on_child_close)
-			child.close()
 		for diag in (
 			self._credentialsDialog,
 			self._smsEntryDialog,
 			self._accountDialog,
-			self._aboutDialog,
 		):
 			if diag is not None:
 				diag.close()
-		self._window.close()
+		qwrappers.WindowWrapper.close(self)
 
 	def destroy(self):
+		qwrappers.WindowWrapper.destroy(self)
 		if self._session.state != self._session.LOGGEDOUT_STATE:
 			self._session.logout()
 
@@ -644,36 +499,15 @@ class MainWindow(object):
 			for settingName, settingValue in tabSettings.iteritems():
 				config.set(sectionName, settingName, settingValue)
 
-	def show(self):
-		self._window.show()
-		for child in self.walk_children():
-			child.show()
-
-	def hide(self):
-		for child in self.walk_children():
-			child.hide()
-		self._window.hide()
-
-	def set_fullscreen(self, isFullscreen):
-		if isFullscreen:
-			self._window.showFullScreen()
-		else:
-			self._window.showNormal()
-		for child in self.walk_children():
-			child.set_fullscreen(isFullscreen)
-
 	def set_orientation(self, isPortrait):
+		qwrappers.WindowWrapper.set_orientation(self, isPortrait)
 		if isPortrait:
 			self._tabWidget.setTabPosition(QtGui.QTabWidget.South)
-			qui_utils.set_window_orientation(self.window, QtCore.Qt.Vertical)
 		else:
 			self._tabWidget.setTabPosition(QtGui.QTabWidget.West)
-			qui_utils.set_window_orientation(self.window, QtCore.Qt.Horizontal)
 		for child in (self._smsEntryDialog, ):
 			if child is not None:
 				child.set_orientation(isPortrait)
-		for child in self.walk_children():
-			child.set_orientation(isPortrait)
 
 	def _initialize_tab(self, index):
 		assert index < self.MAX_TABS, "Invalid tab"
@@ -686,9 +520,12 @@ class MainWindow(object):
 		if self._credentialsDialog is None:
 			import dialogs
 			self._credentialsDialog = dialogs.CredentialsDialog(self._app)
-		username, password = self._credentialsDialog.run(
+		credentials = self._credentialsDialog.run(
 			self._defaultCredentials[0], self._defaultCredentials[1], self.window
 		)
+		if credentials is None:
+			return
+		username, password = credentials
 		self._curentCredentials = username, password
 		self._session.login(username, password)
 
@@ -803,22 +640,6 @@ class MainWindow(object):
 		with qui_utils.notify_error(self._errorLog):
 			self._show_account_dialog()
 
-	@QtCore.pyqtSlot()
-	@QtCore.pyqtSlot(bool)
-	@misc_utils.log_exception(_moduleLogger)
-	def _on_about(self, checked = True):
-		with qui_utils.notify_error(self._errorLog):
-			if self._aboutDialog is None:
-				import dialogs
-				self._aboutDialog = dialogs.AboutDialog(self._app)
-			response = self._aboutDialog.run(self.window)
-
-	@QtCore.pyqtSlot()
-	@QtCore.pyqtSlot(bool)
-	@misc_utils.log_exception(_moduleLogger)
-	def _on_close_window(self, checked = True):
-		self.close()
-
 
 def run():
 	app = QtGui.QApplication([])
